@@ -17,6 +17,16 @@ pub use solana_perf::sigverify::{
 pub struct TransactionSigVerifier {
     recycler: Recycler<TxOffset>,
     recycler_out: Recycler<PinnedVec<u8>>,
+    reject_non_vote: bool,
+}
+
+impl TransactionSigVerifier {
+    pub fn new_reject_non_vote() -> Self {
+        TransactionSigVerifier {
+            reject_non_vote: true,
+            ..TransactionSigVerifier::default()
+        }
+    }
 }
 
 impl Default for TransactionSigVerifier {
@@ -25,13 +35,19 @@ impl Default for TransactionSigVerifier {
         Self {
             recycler: Recycler::warmed(50, 4096),
             recycler_out: Recycler::warmed(50, 4096),
+            reject_non_vote: false,
         }
     }
 }
 
 impl SigVerifier for TransactionSigVerifier {
     fn verify_batch(&self, mut batch: Vec<Packets>) -> Vec<Packets> {
-        sigverify::ed25519_verify(&mut batch, &self.recycler, &self.recycler_out);
+        sigverify::ed25519_verify(
+            &mut batch,
+            &self.recycler,
+            &self.recycler_out,
+            self.reject_non_vote,
+        );
         batch
     }
 }
