@@ -120,10 +120,10 @@ pub struct ValidatorConfig {
     pub fixed_leader_schedule: Option<FixedSchedule>,
     pub wait_for_supermajority: Option<Slot>,
     pub new_hard_forks: Option<Vec<Slot>>,
-    pub trusted_validators: Option<HashSet<Pubkey>>, // None = trust all
-    pub repair_validators: Option<HashSet<Pubkey>>,  // None = repair from all
-    pub gossip_validators: Option<HashSet<Pubkey>>,  // None = gossip with all
-    pub halt_on_trusted_validators_accounts_hash_mismatch: bool,
+    pub known_validators: Option<HashSet<Pubkey>>, // None = trust all
+    pub repair_validators: Option<HashSet<Pubkey>>, // None = repair from all
+    pub gossip_validators: Option<HashSet<Pubkey>>, // None = gossip with all
+    pub halt_on_known_validators_accounts_hash_mismatch: bool,
     pub accounts_hash_fault_injection_slots: u64, // 0 = no fault injection
     pub frozen_accounts: Vec<Pubkey>,
     pub no_rocksdb_compaction: bool,
@@ -178,10 +178,10 @@ impl Default for ValidatorConfig {
             fixed_leader_schedule: None,
             wait_for_supermajority: None,
             new_hard_forks: None,
-            trusted_validators: None,
+            known_validators: None,
             repair_validators: None,
             gossip_validators: None,
-            halt_on_trusted_validators_accounts_hash_mismatch: false,
+            halt_on_known_validators_accounts_hash_mismatch: false,
             accounts_hash_fault_injection_slots: 0,
             frozen_accounts: vec![],
             no_rocksdb_compaction: false,
@@ -588,7 +588,7 @@ impl Validator {
                     genesis_config.hash(),
                     ledger_path,
                     config.validator_exit.clone(),
-                    config.trusted_validators.clone(),
+                    config.known_validators.clone(),
                     rpc_override_health_check.clone(),
                     optimistically_confirmed_bank.clone(),
                     config.send_transaction_service_config.clone(),
@@ -681,6 +681,7 @@ impl Validator {
                     &exit,
                     &cluster_info,
                     snapshot_config.maximum_snapshots_to_retain,
+                    snapshot_config.packager_thread_niceness_adj,
                 );
                 (
                     Some(snapshot_packager_service),
@@ -789,10 +790,10 @@ impl Validator {
             cluster_confirmed_slot_receiver,
             TvuConfig {
                 max_ledger_shreds: config.max_ledger_shreds,
-                halt_on_trusted_validators_accounts_hash_mismatch: config
-                    .halt_on_trusted_validators_accounts_hash_mismatch,
+                halt_on_known_validators_accounts_hash_mismatch: config
+                    .halt_on_known_validators_accounts_hash_mismatch,
                 shred_version: node.info.shred_version,
-                trusted_validators: config.trusted_validators.clone(),
+                known_validators: config.known_validators.clone(),
                 repair_validators: config.repair_validators.clone(),
                 accounts_hash_fault_injection_slots: config.accounts_hash_fault_injection_slots,
                 accounts_db_caching_enabled: config.accounts_db_caching_enabled,
